/*
 * MZViewer - A viewer for MZData format Mass Spectra
 * Copyright (c) 2005 Simon Andrews

 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
package uk.ac.bbsrc.babraham.mzdata.displays;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;

import uk.ac.bbsrc.babraham.mzdata.MZSpectrum;

/**
 * @author Andrewss
 *
 */
public class SingleSpectrumDisplay extends JPanel implements SpectrumSelectionListener {
	
	private MZSpectrum [] spectra;
	private int currentSpectrum = 0;
	private SingleSpectrumDisplayPanel spectrumDisplay;
	private JButton nextButton;
	private JButton childrenButton;
	private JButton prevButton;
	private JButton parentsButton;
	private JLabel textLabel;
	
	public SingleSpectrumDisplay () {
		spectrumDisplay = new SingleSpectrumDisplayPanel();
		JPanel buttonPanel = new JPanel();
		buttonPanel.setBackground(Color.WHITE);
		ClickListener l = new ClickListener(this);
		
		childrenButton = new JButton("Children");
		childrenButton.addActionListener(l);
		childrenButton.setActionCommand("children");
		childrenButton.setEnabled(false);
		buttonPanel.add(childrenButton);
		
		prevButton = new JButton("Prev");
		prevButton.addActionListener(l);
		prevButton.setActionCommand("prev");
		prevButton.setEnabled(false);
		buttonPanel.add(prevButton);

		textLabel = new JLabel("No spectrum available");
		buttonPanel.add(textLabel);
		
		nextButton = new JButton("Next");
		nextButton.addActionListener(l);
		nextButton.setActionCommand("next");
		nextButton.setEnabled(false);
		buttonPanel.add(nextButton);
		
		parentsButton = new JButton("Parents");
		parentsButton.addActionListener(l);
		parentsButton.setActionCommand("parents");
		parentsButton.setEnabled(false);
		buttonPanel.add(parentsButton);
		
		setLayout(new BorderLayout());
		add(buttonPanel,BorderLayout.SOUTH);
		add(spectrumDisplay,BorderLayout.CENTER);
	}
	
	public SingleSpectrumDisplay (MZSpectrum [] spectra) {
		this();
		setSpectra(spectra);
	}

	public void setSpectra (MZSpectrum [] spectra) {
		this.spectra = spectra;
		if (spectra.length > 0) {
			spectrumDisplay.setSpectrum(spectra[0]);
			currentSpectrum = 0;
			textLabel.setText("Spectrum 1 of "+spectra.length);
			resetButtons();
		}
	}
	
	public void spectrumSelected(MZSpectrum spectrum) {
        // We need to know if this spectrum is part of
        // a series of children.
        
        if (spectrum != null && spectrum.hasParents()) {
            // Spectrum could have siblings
            // Add all the children of the first parent (in the absence of any better idea
            setSpectra((spectrum.getParents())[0].getChildren());

            // Then make sure this one is selected.
            spectrumDisplay.setSpectrum(spectrum);
            currentSpectrum = (spectrum.getParents())[0].getChildIndex(spectrum);
            resetButtons();
        }
        
        else {
            // If there's no parent then we just have one
            // spectrum to deal with.
            setSpectra(new MZSpectrum [] {spectrum});
        }
	}
	
	private void resetButtons () {
	    if (spectra == null || spectra[currentSpectrum] == null) {
	        nextButton.setEnabled(false);
	        prevButton.setEnabled(false);
	        childrenButton.setEnabled(false);
	        parentsButton.setEnabled(false);
            textLabel.setText("No spectrum available");
            return;
	    }
        else {
            textLabel.setText("Spectrum "+(currentSpectrum+1)+" of "+spectra.length);
        }
		if (spectra.length == currentSpectrum+1) {
			nextButton.setEnabled(false);
		}
		else {
		    nextButton.setEnabled(true);
		}
		if (currentSpectrum == 0) {
			prevButton.setEnabled(false);
		}
		else {
		    prevButton.setEnabled(true);
		}
	    if (spectra[currentSpectrum].hasChildren()) {
	        childrenButton.setEnabled(true);
	    }
	    else {
	        childrenButton.setEnabled(false);
	    }
	    if (spectra[currentSpectrum].hasParents()) {
	        parentsButton.setEnabled(true);
	    }
	    else {
	        parentsButton.setEnabled(false);
	    }
	}


	
	private class ClickListener implements ActionListener {
		Component c;
		public ClickListener (Component c) {
			this.c = c;
		}

		public void actionPerformed(ActionEvent ae) {
			String command = ae.getActionCommand();
			
			if (command.equals("next")) {
				++currentSpectrum;
                spectrumDisplay.setSpectrum(spectra[currentSpectrum]);
			}
			else if (command.equals("prev")) {
				--currentSpectrum;
				spectrumDisplay.setSpectrum(spectra[currentSpectrum]);
			}
			else if (command.equals("children")) {
                MZSpectrum [] children = spectra[currentSpectrum].getChildren();
                JPopupMenu popup = new JPopupMenu("Children");
                for (int i=0;i<children.length;i++) {
                    JMenuItem item = new JMenuItem(""+children[i].getParentMZ());
                    item.setActionCommand(""+i);
                    item.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent ie) {
                            setSpectra(spectra[currentSpectrum].getChildren());
                            currentSpectrum = Integer.parseInt(ie.getActionCommand());
                            spectrumDisplay.setSpectrum(spectra[currentSpectrum]);
                            resetButtons();
                        }
                    });
                    popup.add(item);
                }
                // Make the popup show above the child button
                popup.show(childrenButton,0,(-20 * spectra[currentSpectrum].getChildren().length));                  
			}
			else if (command.equals("parents")) {
			    setSpectra(spectra[currentSpectrum].getParents());
			}

			resetButtons();
		}
	}
}
